package com.istnetworks.hivesdk.presentation.spinnerquestion

import androidx.core.os.bundleOf
import androidx.fragment.app.testing.launchFragmentInContainer
import androidx.test.espresso.Espresso.onView
import androidx.test.espresso.assertion.ViewAssertions.matches
import androidx.test.espresso.matcher.ViewMatchers.isDisplayed
import androidx.test.espresso.matcher.ViewMatchers.withId
import androidx.test.ext.junit.runners.AndroidJUnit4
import com.istnetworks.hivesdk.R
import com.istnetworks.hivesdk.data.models.SurveyOptions
import com.istnetworks.hivesdk.data.models.response.Question
import com.istnetworks.hivesdk.data.models.response.Survey
import com.istnetworks.hivesdk.data.repository.HiveSDKRepository
import com.istnetworks.hivesdk.data.utils.QuestionType
import com.istnetworks.hivesdk.domain.usecases.SkipLogicHandler
import com.istnetworks.hivesdk.domain.usecases.Validator
import com.istnetworks.hivesdk.getSurveyTheme
import com.istnetworks.hivesdk.presentation.viewmodel.HiveSDKViewModel
import io.mockk.every
import io.mockk.mockk
import io.mockk.spyk
import org.junit.Before
import org.junit.Test
import org.junit.runner.RunWith

@RunWith(AndroidJUnit4::class)
class SpinnerQuestionFragmentTest {
    lateinit var viewModel: HiveSDKViewModel
    lateinit var repo: HiveSDKRepository
    lateinit var skipLogicHandler: SkipLogicHandler
    lateinit var validator: Validator
    lateinit var survey: Survey

    @Before
    fun setup() {
        repo = mockk()
        skipLogicHandler = mockk()
        validator = mockk()
        viewModel = spyk(HiveSDKViewModel(repo,skipLogicHandler,validator))
    }

    @Test
    fun test_hasSpinner() {
        val q = Question("s", 1, "t", false, QuestionType.SlideQuestion.value)

        every { viewModel.findQuestion(1) } returns q
        every { viewModel.getSurveyTheme() } returns getSurveyTheme()
        val fragmentArgs = bundleOf("pos" to 1)
        val scenario = launchFragmentInContainer<SpinnerQuestionFragment>(fragmentArgs)

        onView(withId(R.id.hve_sp_answers)).check(matches(isDisplayed()))
    }

}
