package com.istnetworks.hivesdk.domain.usecases

import com.istnetworks.hivesdk.data.models.QuestionResponses
import com.istnetworks.hivesdk.data.models.SkipLogic
import com.istnetworks.hivesdk.data.models.response.Question
import com.istnetworks.hivesdk.data.utils.QuestionType

class SkipLogicHandler() {
    private var skipLogic: List<SkipLogic>? = null
    private var questionList: MutableList<Question>? = null

    fun setData(skipLogic: List<SkipLogic>?, questionList: MutableList<Question>?) {
        this.skipLogic = skipLogic
        this.questionList = questionList
    }

    fun getSkipToQuestionPosition(currentQuestionPosition: Int, response: QuestionResponses?): Int {
        val question = findQuestion(currentQuestionPosition)
        if (!hasSkipLogic(question?.surveyQuestionGUID))
            return currentQuestionPosition.plus(1)

        val questionTo =
            when (question?.questionType) {
                in QuestionType.TextInput.value..QuestionType.URLInput.value,
                QuestionType.MultipleChoiceQuestion.value,
                QuestionType.ImageMCQ.value,
                QuestionType.DateQuestion.value -> {
                    getSkipToByQuestionGuid(question)
                }

                QuestionType.ListQuestion.value,
                QuestionType.SingleChoice.value,
                QuestionType.ImageSingleChoice.value,
                QuestionType.CSAT.value -> {
                    getSkipToByQuestionGuidAndChoiceGuid(
                        question.surveyQuestionGUID,
                        response?.choiceGUID
                    )
                }

                QuestionType.SlideQuestion.value,
                QuestionType.StarQuestion.value,
                QuestionType.Emoji.value,
                QuestionType.NPS.value -> {
                    getSkipToByNumberAndQuestionGuid(question, response?.numberResponse)
                }
                else -> {
                    null
                }
            }

        return questionList?.indexOfFirst { it.surveyQuestionGUID!! == questionTo } ?: -1
    }

    fun hasSkipLogic(qGuid: String?): Boolean {
        return skipLogic?.any { it.questionGUID.equals(qGuid) } ?: false
    }

    private fun getSkipToByQuestionGuid(question: Question?): String? {
        return skipLogic?.findLast {
            it.questionGUID.equals(question?.surveyQuestionGUID)
        }?.skipToQuestionGUID
    }

    fun getSkipToByNumberAndQuestionGuid(question: Question?, numberResponse: Int?): String? {
        return skipLogic?.find {
            it.questionGUID.equals(question?.surveyQuestionGUID) &&
                    numberResponse in it.minValue!!..it.maxValue!!
        }?.skipToQuestionGUID
    }

    private fun getSkipToByQuestionGuidAndChoiceGuid(questionGuid: String?, choiceGuid: String?): String? {
        return skipLogic?.find {
            it.questionGUID.equals(questionGuid) && choiceGuid == it.qChoiceGUID
        }?.skipToQuestionGUID
    }


    private fun findQuestion(position: Int?): Question? {
        if (position == null || position == -1) return null
        return questionList?.get(position)
    }
}