package com.istnetworks.hivesdk.presentation.BottomSheet

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.fragment.app.activityViewModels
import androidx.viewpager2.widget.ViewPager2
import com.istnetworks.hivesdk.data.repository.HiveSDKRepositoryImpl
import com.istnetworks.hivesdk.data.utils.Preferences
import com.istnetworks.hivesdk.data.utils.extensions.hide
import com.istnetworks.hivesdk.data.utils.extensions.onClick
import com.istnetworks.hivesdk.data.utils.extensions.show
import com.istnetworks.hivesdk.databinding.FragmentBottomSheetBinding
import com.istnetworks.hivesdk.databinding.FragmentMainBinding
import com.istnetworks.hivesdk.presentation.interfaces.IsRequiredInterface
import com.istnetworks.hivesdk.presentation.interfaces.SubmitButtonInterface
import com.istnetworks.hivesdk.presentation.interfaces.ValidationErrorInterface
import com.istnetworks.hivesdk.presentation.introAndThankYou.ThankYouFragment
import com.istnetworks.hivesdk.presentation.mainfragment.adapter.HorizontalPagerAdapter
import com.istnetworks.hivesdk.presentation.surveyExtension.cardsBackground
import com.istnetworks.hivesdk.presentation.surveyExtension.setSliderColor
import com.istnetworks.hivesdk.presentation.surveyExtension.surveyLogoStyle
import com.istnetworks.hivesdk.presentation.surveyExtension.surveyTitleStyle
import com.istnetworks.hivesdk.presentation.viewmodel.HiveSDKViewModel
import com.istnetworks.hivesdk.presentation.viewmodel.factory.HiveSDKViewModelFactory


class BottomSheetFragment : Fragment() {

    private lateinit var binding: FragmentBottomSheetBinding
    lateinit var onPageChangeCallback: ViewPager2.OnPageChangeCallback
    lateinit var horizontalPagerAdapter: HorizontalPagerAdapter

    private val viewModel: HiveSDKViewModel by activityViewModels {
        HiveSDKViewModelFactory(
            HiveSDKRepositoryImpl(sharedPref = Preferences(requireActivity()))
        )
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        binding = FragmentBottomSheetBinding.inflate(inflater)
        initializeViewPager()
        bindViews()
        onClickActions()
        setupProgressSlider()
        listenToViewPagerChanges()
        observeViewModel()
        return binding.root
    }


    private fun observeViewModel() {
        viewModel.updateProgressSliderLD.observe(viewLifecycleOwner, {
            if (it > -1)
                if (it >= binding.hveHeader.hveSliderProgress.valueTo)
                    onProgressBarCompleted()
                else{
                    binding.hveHeader.hveSliderProgress.value = it
                    setProgressBarFocusColor(viewModel.getSurveyTheme()?.focusColor)
                }
        })
        viewModel.getIsValidationErrorLD().observe(viewLifecycleOwner, {
            val f =
                horizontalPagerAdapter.getFragmentByPosition(binding.hveViewPager.currentItem)
            val questionType =
                viewModel.findQuestion(binding.hveViewPager.currentItem)?.questionType
            if (it.second == true) {
                (f as ValidationErrorInterface).showNotValidError(questionType)
            }else
            {
                (f as ValidationErrorInterface).hideNotValidError(questionType)
            }

        })
        viewModel.getIsRequiredErrorLD().observe(viewLifecycleOwner, {
            val f =
                horizontalPagerAdapter.getFragmentByPosition(binding.hveViewPager.currentItem)
            if (it.second== true) {
                (f as IsRequiredInterface).showIsRequiredError()
            } else {
                (f as IsRequiredInterface).hideIsRequiredError()
            }
        })

        viewModel.enableNextButtonLD.observe(viewLifecycleOwner, {
            binding.hveIvNext.isEnabled = it
            binding.hveIvNext.alpha = if (it) 1f else .6f
        })

        viewModel.showSubmitButtonLD.observe(viewLifecycleOwner, {
            val f =
                horizontalPagerAdapter.getFragmentByPosition(binding.hveViewPager.currentItem)
            if (f is SubmitButtonInterface)
                if (it.second == true)
                    (f as SubmitButtonInterface).showSubmitButton()
                else
                    (f as SubmitButtonInterface).hideSubmitButton()

        })
        viewModel.saveSurveyResponseLD.observe(viewLifecycleOwner, {
            showThankYouDialog()
        })
        viewModel.resetNextViewPagerFragmentsLD.observe(viewLifecycleOwner) {
            removeNextFragmentsResponses(it)
            horizontalPagerAdapter.resetFragmentsByPositions(it)
            binding.hveViewPager.invalidate()
            // binding.hveViewPager.adapter=horizontalPagerAdapter
        }
    }
    private fun removeNextFragmentsResponses(positions: List<Int>) {
        val questions = viewModel.survey?.questions
        positions.forEach {
            val surveyQuestionGUID = questions?.get(it)?.surveyQuestionGUID
            viewModel.removeResponseByQuestionGuid(surveyQuestionGUID)
        }
    }
    private fun showThankYouDialog() {
        val beginTransaction = activity?.supportFragmentManager?.beginTransaction()
        beginTransaction?.remove(this)
        beginTransaction?.add(android.R.id.content, ThankYouFragment.newInstance(), "")?.commit()

    }
    private fun onProgressBarCompleted() {
        binding.hveHeader.hveSliderProgress.value = binding.hveHeader.hveSliderProgress.valueTo
        setProgressBarFocusColor("00ff00")
    }

    private fun setProgressBarFocusColor(color: String?) {
        binding.hveHeader.hveSliderProgress.setSliderColor(
            viewModel.getSurveyTheme()?.surveyBackgroundColor!!,
            color!!
        )
    }

    private fun setupProgressSlider() {
        if (viewModel.hasProgressBar() == true) {
            binding.hveHeader.hveSliderProgress.isEnabled = false
            binding.hveHeader.hveSliderProgress.thumbRadius=0
            binding.hveHeader.hveSliderProgress.show()
            binding.hveHeader.hveSliderProgress.valueFrom = 0f
            binding.hveHeader.hveSliderProgress.stepSize = 1f
            binding.hveHeader.hveSliderProgress.valueTo = viewModel.survey?.questions?.size?.toFloat() ?: 1f
            setProgressBarFocusColor(viewModel.getSurveyTheme()?.focusColor)

        } else {
            binding.hveHeader.hveSliderProgress.hide()
        }
    }

    private fun listenToViewPagerChanges() {
        onPageChangeCallback = object : ViewPager2.OnPageChangeCallback() {
            override fun onPageSelected(position: Int) {
                binding.hveIvPrevious.isEnabled = position > 0
                viewModel.updateNextArrowEnablingOnFragmentChange(position)

            }
        }
        binding.hveViewPager.registerOnPageChangeCallback(onPageChangeCallback)
    }


    private fun onClickActions() {
        binding.hveIvNext.onClick {
            onNextArrowClicked()
        }
        binding.hveIvPrevious.onClick {
            val newPosition =
                viewModel.getAndPopPreviousPosition()
            binding.hveViewPager.setCurrentItem(newPosition,false)
        }

        binding.hveHeader.hveIvClose.onClick {
            requireActivity().finish()
        }
    }

    private fun onNextArrowClicked() {
        if (viewModel.validateAnswer(binding.hveViewPager.currentItem)) {
            navigateToNextQuestion()
        }
    }

    private fun navigateToNextQuestion() {
        val newPosition =
            viewModel.getSkipToQuestionPosition(binding.hveViewPager.currentItem)
            viewModel.pushToPreviousQuestionsStack(binding.hveViewPager.currentItem)
        binding.hveViewPager.setCurrentItem(newPosition,false)
    }

    private fun bindViews() {
        binding.hveHeader.hveIvIcon.surveyLogoStyle(viewModel.getSurveyTheme()?.surveyLogoStyle!!)
        binding.hveHeader.tvSurveyTitle.text = viewModel.survey?.title
        binding.hveHeader.tvSurveyTitle.surveyTitleStyle(viewModel.getSurveyTheme()?.surveyTitleStyle)
        binding.hveHeader.hveClHeader.cardsBackground(requireContext(), viewModel.surveyBackgroundColor())
        binding.hveClQuestionHolder.cardsBackground(
            requireContext(),
            viewModel.surveyBackgroundColor()
        )
        binding.hveHeader.hveIvClose.visibility =
            if (viewModel.getSurveyOptions()?.enableCloseButton == true) {
                View.VISIBLE
            } else {
                View.GONE
            }

    }

    private fun initializeViewPager() {
        binding.hveViewPager.isUserInputEnabled = false
        horizontalPagerAdapter = HorizontalPagerAdapter(this)
        horizontalPagerAdapter.setData(viewModel.survey?.questions ?: listOf())
        binding.hveViewPager.apply {
            offscreenPageLimit = 1
            orientation = ViewPager2.ORIENTATION_HORIZONTAL
        }
        binding.hveViewPager.adapter = horizontalPagerAdapter
    }

    override fun onDestroyView() {
        super.onDestroyView()
        binding.hveViewPager.unregisterOnPageChangeCallback(onPageChangeCallback)
    }
}