package com.istnetworks.hivesdk.presentation.datepickerquestion

import android.app.DatePickerDialog
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.annotation.Keep
import androidx.annotation.NonNull
import androidx.core.content.ContextCompat
import androidx.core.os.bundleOf
import com.istnetworks.hivesdk.R
import com.istnetworks.hivesdk.data.models.response.Question
import com.istnetworks.hivesdk.data.models.response.toQuestionResponse
import com.istnetworks.hivesdk.data.utils.extensions.hide
import com.istnetworks.hivesdk.data.utils.extensions.onClick
import com.istnetworks.hivesdk.data.utils.extensions.show
import com.istnetworks.hivesdk.databinding.FragmentDatePickerQuestionBinding
import com.istnetworks.hivesdk.presentation.BaseQuestionFragment
import com.istnetworks.hivesdk.presentation.interfaces.FullScreenQuestionNumberSetter
import com.istnetworks.hivesdk.presentation.interfaces.IsRequiredInterface
import com.istnetworks.hivesdk.presentation.interfaces.SubmitButtonInterface
import java.text.SimpleDateFormat
import java.util.*


const val ARG_POSITION = "pos"

class DatePickerQuestionFragment : BaseQuestionFragment(), IsRequiredInterface,
    SubmitButtonInterface, FullScreenQuestionNumberSetter {
    private lateinit var binding: FragmentDatePickerQuestionBinding
    private var selectedQuestion: Question? = null
    private val position: Int? by lazy { arguments?.getInt(ARG_POSITION, -1) }
    private var questionNumber: Int = -1
    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        binding = FragmentDatePickerQuestionBinding.inflate(inflater)
        selectedQuestion = viewModel.findQuestion(position)
        stylingQuestionTitle(binding.tvQuestionTitle)
        initSubmitBtn()
        onClickActions()
        handleDatePickerField()
        observeViewModel()
        return binding.root
    }

    private fun observeViewModel() {

    }

    override fun onResume() {
        super.onResume()
        binding.root.requestLayout()
        bindQuestion()
        checkInputsAndIsRequiredError()
    }

    private fun checkInputsAndIsRequiredError(){
        val isRequired = selectedQuestion?.isRequired

        if(binding.hveTietDate.text?.isEmpty() == true
            && isRequired== true)
        {
            binding.tvErrorMessage.visibility = View.GONE
        }

    }

    private fun bindQuestion() {
        binding.tvQuestionTitle.text =
            "${context?.getString(R.string.questionLabel)}${getQNumber()}. ${selectedQuestion?.title}"
    }

    private fun getQNumber(): Int {
        return if (viewModel.isFullScreen())
            questionNumber
        else
            viewModel.getPopupQuestionNumber()
    }

    private fun initSubmitBtn() {
        viewModel.setSubmitButtonBasedOnPosition(binding.hveBtnSubmit, position)
    }

    private fun handleDatePickerField() {

        val c = Calendar.getInstance()
        val year = c.get(Calendar.YEAR)
        val month = c.get(Calendar.MONTH)
        val day = c.get(Calendar.DAY_OF_MONTH)
        binding.hveTietDate.background =
            activity?.let { it1 ->
                ContextCompat.getDrawable(
                    it1,
                    R.drawable.bg_free_input_rounded_border
                )
            }
        binding.hveTietDate.setTextColor(ContextCompat.getColor(requireContext(), R.color.navyBlue))
        binding.hveTietDate.onClick {
            //   binding.hveTietDate.disableForSecond()

            val dialog = DatePickerDialog(
                requireContext(), { _, year, month, dayOfMonth ->

                    val sdf = SimpleDateFormat("dd MMMM,yyyy",Locale.getDefault())
                    c.set(year, month, dayOfMonth)
                    val dateString: String = sdf.format(c.getTime())
                    binding.hveTietDate.setText(dateString)
                    saveAnswer(binding.hveTietDate.text.toString())
                }, year, month, day - 1
            )

            dialog.show()
        }
    }

    private fun saveAnswer(answerText: String) {
        viewModel.updateQuestionResponsesList(
            selectedQuestion?.toQuestionResponse(
                answerText,
                0
            )
        )
    }


    @Keep
    companion object {
        @JvmStatic
        fun getInstance(@NonNull position: Int): DatePickerQuestionFragment {
            val f = DatePickerQuestionFragment()
            f.arguments = bundleOf(ARG_POSITION to position)
            return f
        }

    }

    private fun onClickActions() {

    }

    override fun showIsRequiredError() {
        binding.tvErrorMessage.show()
        updatePagerHeight(binding.root)

    }

    override fun hideIsRequiredError() {
        binding.tvErrorMessage.hide()
        updatePagerHeight(binding.root)

    }

    override fun showSubmitButton() {
        binding.hveBtnSubmit.show()
        updatePagerHeight(binding.root)
    }

    override fun hideSubmitButton() {
        binding.hveBtnSubmit.hide()
        updatePagerHeight(binding.root)
    }

    override fun setQuestionNumber(num: Int) {
        questionNumber = num
    }
}