package com.istnetworks.hivesdk.presentation.introAndThankYou

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import com.istnetworks.hivesdk.R
import com.istnetworks.hivesdk.data.models.response.styles.WelcomeMessage
import com.istnetworks.hivesdk.data.utils.extensions.onClick
import com.istnetworks.hivesdk.databinding.FragmentIntroBinding
import com.istnetworks.hivesdk.presentation.BaseQuestionFragment
import com.istnetworks.hivesdk.presentation.BottomSheet.BottomSheetFragment
import com.istnetworks.hivesdk.presentation.fullScreen.VerticalMainFragment
import com.istnetworks.hivesdk.presentation.mainfragment.MainFragment
import com.istnetworks.hivesdk.presentation.surveyExtension.submitButtonStyle
import com.istnetworks.hivesdk.presentation.surveyExtension.welcomeSubTitleStyle
import com.istnetworks.hivesdk.presentation.surveyExtension.welcomeTitleStyle

class IntroFragment : BaseQuestionFragment()  {
    private val welcomeStyle : WelcomeMessage? by lazy{ viewModel.getSurveyTheme()?.welcomeMessage}
    private lateinit var binding: FragmentIntroBinding

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        binding = FragmentIntroBinding.inflate(inflater,container,false)
        stylingViews()
        addValuesToViews()
        attachActionToClick()
        return binding.root
    }

    private fun stylingViews() {
        binding.hveHelloTitle.welcomeTitleStyle(welcomeStyle?.titleStyle)
        binding.hveHelloSubtitle.welcomeSubTitleStyle(welcomeStyle?.subTitleStyle)
        binding.hveTakeSurvey.submitButtonStyle(viewModel.getSurveyTheme()?.submitButton)
    }

    private fun attachActionToClick(){
        binding.hveTakeSurvey.onClick {
            when  {
                viewModel.isFullScreen() -> showCardDialog(VerticalMainFragment())
                viewModel.isBottomSheet() -> showCardDialog(BottomSheetFragment())
                viewModel.isPopup() -> showCardDialog(MainFragment())
            }

        }

        binding.hveIvClose.onClick {
            requireActivity().finish()
        }
    }

    private fun addValuesToViews(){
        binding.hveHelloTitle.text = welcomeStyle?.title ?: getString(R.string.userName)
        binding.hveHelloSubtitle.text= welcomeStyle?.subTitle
    }

    companion object {

        @JvmStatic
        fun newInstance() =IntroFragment()
    }

    private fun showCardDialog(f: Fragment) {
        val beginTransaction = activity?.supportFragmentManager?.beginTransaction()
        beginTransaction?.remove(this)
        beginTransaction?.add(android.R.id.content, f, "")?.commit()

    }
}