package com.istnetworks.hivesdk.presentation.mainfragment.adapter

import androidx.fragment.app.Fragment
import androidx.lifecycle.Lifecycle
import androidx.viewpager2.adapter.FragmentStateAdapter
import androidx.viewpager2.widget.ViewPager2
import com.istnetworks.hivesdk.data.models.response.Question
import com.istnetworks.hivesdk.data.utils.QuestionType
import com.istnetworks.hivesdk.presentation.datepickerquestion.DatePickerQuestionFragment
import com.istnetworks.hivesdk.presentation.emojis.EmojiFragment
import com.istnetworks.hivesdk.presentation.freeinputs.FreeInputsFragment
import com.istnetworks.hivesdk.presentation.multiImageChoice.MultipleImageChoiceFragment
import com.istnetworks.hivesdk.presentation.multipleChoices.MultipleChoicesFragment
import com.istnetworks.hivesdk.presentation.nps.NpsFragment
import com.istnetworks.hivesdk.presentation.rating.RatingFragment
import com.istnetworks.hivesdk.presentation.singleChoice.SingleChoiceFragment
import com.istnetworks.hivesdk.presentation.singleImageChoice.SingleImageChoiceFragment
import com.istnetworks.hivesdk.presentation.sliderquestion.SliderQuestionFragment
import com.istnetworks.hivesdk.presentation.spinnerquestion.SpinnerQuestionFragment

/**
 * Created by khairy on ن, 06/ماي/2019 at 03:15 م.
 *
 */
class HorizontalPagerAdapter(f: Fragment) : FragmentStateAdapter(f.childFragmentManager,f.lifecycle) {

    private val mFragmentList: MutableList<Fragment> = mutableListOf()
    private var mQuestions: List<Question?> = listOf()
    private var mPageIds: MutableList<Long> = mutableListOf()
    fun setData(questions: List<Question?>) {
        mFragmentList.clear()
        mQuestions = questions
        addFragmentsFromQuestions(questions)
        mPageIds = mFragmentList.map { it.hashCode().toLong() }.toMutableList()
    }

    fun getFragmentByPosition(fragmentPosition: Int): Fragment {
        return mFragmentList.get(fragmentPosition)
    }

    override fun getItemId(position: Int): Long {
        return mFragmentList[position].hashCode().toLong()
    }

    override fun containsItem(itemId: Long): Boolean {
        return mPageIds.contains(itemId)
    }

    private fun getFragmentFromType(it: Int?, questionPosition: Int): Fragment {
        return when (it) {
            QuestionType.MultipleChoiceQuestion.value -> MultipleChoicesFragment.newInstance(
                questionPosition
            )
            QuestionType.ListQuestion.value -> SpinnerQuestionFragment.getInstance(questionPosition)
            QuestionType.DateQuestion.value -> DatePickerQuestionFragment.getInstance(
                questionPosition
            )
            QuestionType.SlideQuestion.value -> SliderQuestionFragment.getInstance(questionPosition)
            QuestionType.StarQuestion.value -> RatingFragment.getInstance(questionPosition)
            QuestionType.NPS.value -> NpsFragment.getInstance(questionPosition)
            QuestionType.TextInput.value -> FreeInputsFragment.getInstance(questionPosition)
            QuestionType.NumberInput.value -> FreeInputsFragment.getInstance(questionPosition)
            QuestionType.EmailInput.value -> FreeInputsFragment.getInstance(questionPosition)
            QuestionType.PhoneNumberInput.value -> FreeInputsFragment.getInstance(questionPosition)
            QuestionType.PostalCodeInput.value -> FreeInputsFragment.getInstance(questionPosition)
            QuestionType.URLInput.value -> FreeInputsFragment.getInstance(questionPosition)
            QuestionType.TextInput.value -> FreeInputsFragment.getInstance(questionPosition)
            QuestionType.NumberInput.value -> FreeInputsFragment.getInstance(questionPosition)
            QuestionType.EmailInput.value -> FreeInputsFragment.getInstance(questionPosition)
            QuestionType.PhoneNumberInput.value -> FreeInputsFragment.getInstance(questionPosition)
            QuestionType.PostalCodeInput.value -> FreeInputsFragment.getInstance(questionPosition)
            QuestionType.URLInput.value -> FreeInputsFragment.getInstance(questionPosition)
            QuestionType.TextInput.value -> FreeInputsFragment.getInstance(questionPosition)
            QuestionType.NumberInput.value -> FreeInputsFragment.getInstance(questionPosition)
            QuestionType.EmailInput.value -> FreeInputsFragment.getInstance(questionPosition)
            QuestionType.PhoneNumberInput.value -> FreeInputsFragment.getInstance(questionPosition)
            QuestionType.PostalCodeInput.value -> FreeInputsFragment.getInstance(questionPosition)
            QuestionType.URLInput.value -> FreeInputsFragment.getInstance(questionPosition)
            QuestionType.SingleChoice.value -> SingleChoiceFragment.newInstance(questionPosition)
            QuestionType.Emoji.value -> EmojiFragment.getInstance(questionPosition)
            QuestionType.ImageMCQ.value -> MultipleImageChoiceFragment.getInstance(questionPosition)
            QuestionType.ImageSingleChoice.value -> SingleImageChoiceFragment.getInstance(questionPosition)
            QuestionType.CSAT.value -> SingleChoiceFragment.newInstance(questionPosition)
            null ->SpinnerQuestionFragment.getInstance(questionPosition)
            else -> SpinnerQuestionFragment.getInstance(questionPosition)
        }
    }


    override fun getItemCount(): Int {
        return mFragmentList.size
    }


    override fun createFragment(position: Int): Fragment {
        return mFragmentList[position]
    }

    fun resetFragmentsByPositions(positions: List<Int>) {
        positions.forEach {
            val newFragment = getFragmentFromType(mQuestions[it]?.questionType, it)
            mFragmentList[it] = newFragment
            mPageIds[it] = newFragment.id.toLong()
            notifyItemChanged(it)
        }

    }

    private fun addFragmentsFromQuestions(subList: List<Question?>) {
        mFragmentList.addAll(subList.mapIndexed { _, it ->
            getFragmentFromType(
                it?.questionType,
               getQuestionPosition(it)
            )
        })
    }
    private fun getQuestionPosition(question: Question?):Int{
       return mQuestions.indexOf(question)
    }
}