package com.istnetworks.hivesdk.presentation.surveyExtension

import android.content.Context
import android.content.res.ColorStateList
import android.graphics.Color
import android.graphics.Paint
import android.graphics.drawable.GradientDrawable
import android.os.Build
import android.util.LayoutDirection
import android.util.Patterns
import android.view.View
import android.view.ViewGroup
import android.widget.*
import androidx.core.content.ContextCompat
import com.google.android.material.button.MaterialButton
import com.google.android.material.slider.Slider
import com.istnetworks.hivesdk.R
import com.istnetworks.hivesdk.data.models.response.styles.*
import com.squareup.picasso.Picasso

private const val URL_PATTERN = "(https?:\\/\\/(?:www\\.|(?!www))[a-zA-Z0-9][a-zA-Z0-9-]+[a-zA-Z0-9]\\.[^\\s]{2,}|www\\.[a-zA-Z0-9][a-zA-Z0-9-]+[a-zA-Z0-9]\\.[^\\s]{2,}|https?:\\/\\/(?:www\\.|(?!www))[a-zA-Z0-9]+\\.[^\\s]{2,}|www\\.[a-zA-Z0-9]+\\.[^\\s]{2,})"

/**
 * Survey title style
 */
fun TextView.surveyTitleStyle(titleStyle: TitleStyle?) {
   setTitleStyle(this,titleStyle)
}

/**
 * Welcome screen title style
 */
fun TextView.welcomeTitleStyle(titleStyle: TitleStyle?) {
    setTitleStyle(this,titleStyle)
}

/**
 * Welcome screen SubTitle Style
 */
fun TextView.welcomeSubTitleStyle(titleStyle: TitleStyle?) {
    setTitleStyle(this,titleStyle)
}

/**
 * Welcome screen title style
 */
fun TextView.thankYouTitleStyle(titleStyle: TitleStyle?) {
   setTitleStyle(this,titleStyle)
}

/**
 * Welcome screen SubTitle Style
 */
fun TextView.thankYouSubTitleStyle(titleStyle: TitleStyle?) {
    setTitleStyle(this,titleStyle)
}
/**
 * Question title style
 */
fun TextView.questionTitleStyle(titleStyle: TitleStyle?) {
    setTextColor(Color.parseColor("#" + titleStyle?.fontColor))
    typeface = getFontTypeface(
        titleStyle?.fontBold!!, titleStyle.fontItalic!!,
        titleStyle.fontFamily!!
    )
    val sp: Float =
        titleStyle.fontSize?.replace("px", "")?.toFloat()
            ?: 12f / resources.displayMetrics.scaledDensity + 0.5f

    textSize = sp

    if (titleStyle.fontUnderline!!) {
        paintFlags = Paint.UNDERLINE_TEXT_FLAG
    }
}

/**
 * subTitleStyle
 */
fun TextView.subTitleStyle(subtitleStyle: TitleStyle?) {
    setTitleStyle(this,subtitleStyle)
}

/**
 * multiChoice style
 */
fun CheckBox.multiChoiceStyle(questionChoicesStyle: TitleStyle,focusColor:String,
                                themeColor:String)
{
    setTitleStyle(this,questionChoicesStyle)
    background = colorStateDrawable(themeColor,focusColor,80f)
    val colorStateList = ColorStateList(
        arrayOf(
            intArrayOf(-android.R.attr.state_checked),
            intArrayOf(android.R.attr.state_checked)
        ), intArrayOf(
            Color.parseColor("#ffffff")//disabled
            , Color.parseColor("#ffffff") //enabled
        )
    )
    setPadding(64,0,64,0)
    buttonTintList = colorStateList
    layoutDirection = if (isRtl(this.context))
    {
        View.LAYOUT_DIRECTION_LTR
    }
    else
    {
        View.LAYOUT_DIRECTION_RTL
    }
}

/**
 * multiChoice Image style
 */
fun CheckBox.multiChoiceImageStyle(questionChoicesStyle: TitleStyle,focusColor:String,
                              themeColor:String)
{
    setTitleStyle(this,questionChoicesStyle)
    background = colorStateDrawable(themeColor,focusColor,30f)
    val colorStateList = ColorStateList(
        arrayOf(
            intArrayOf(-android.R.attr.state_checked),
            intArrayOf(android.R.attr.state_checked)
        ), intArrayOf(
            Color.parseColor("#ffffff")//disabled
            , Color.parseColor("#ffffff") //enabled
        )
    )
    setPadding(64,0,64,0)
    buttonTintList = colorStateList
    layoutDirection = if (isRtl(this.context))
    {
        View.LAYOUT_DIRECTION_LTR
    }
    else
    {
        View.LAYOUT_DIRECTION_RTL
    }
}

fun RadioButton.singleChoiceStyle(questionChoicesStyle: TitleStyle,focusColor:String,
                                  themeColor:String)
{
    setTitleStyle(this,questionChoicesStyle)
    background = colorStateDrawable(themeColor,focusColor,80f)
    val params = LinearLayout.LayoutParams(
        ViewGroup.LayoutParams.MATCH_PARENT
        , ViewGroup.LayoutParams.WRAP_CONTENT)
    params.topMargin = 24
    setPadding(64,-32,64,-32)
    layoutParams = params

    val colorStateList = ColorStateList(
        arrayOf(
            intArrayOf(-android.R.attr.state_checked),
            intArrayOf(android.R.attr.state_checked)
        ), intArrayOf(
            Color.parseColor("#ffffff") //disabled
            , Color.parseColor("#ffffff") //enabled
        )
    )

    buttonTintList = colorStateList
    layoutDirection = if (isRtl(this.context))
    {
        View.LAYOUT_DIRECTION_LTR
    }
    else
    {
        View.LAYOUT_DIRECTION_RTL
    }
}

/**
 * Singlechoice Image Style
 */
fun RadioButton.singleChoiceImageStyle(questionChoicesStyle: TitleStyle,focusColor:String,
                                  themeColor:String)
{
    setTitleStyle(this,questionChoicesStyle)
    background = colorStateDrawable(themeColor,focusColor,30f)
    val params = LinearLayout.LayoutParams(
        ViewGroup.LayoutParams.MATCH_PARENT
        , ViewGroup.LayoutParams.WRAP_CONTENT)
    params.topMargin = 24
    setPadding(64,-32,64,-32)
    layoutParams = params

    val colorStateList = ColorStateList(
        arrayOf(
            intArrayOf(-android.R.attr.state_checked),
            intArrayOf(android.R.attr.state_checked)
        ), intArrayOf(
            Color.parseColor("#ffffff") //disabled
            , Color.parseColor("#ffffff") //enabled
        )
    )

    buttonTintList = colorStateList
    layoutDirection = if (isRtl(this.context))
    {
        View.LAYOUT_DIRECTION_LTR
    }
    else
    {
        View.LAYOUT_DIRECTION_RTL
    }
}
/**
 * start screen message
 */
fun TextView.startScreenMsgStyle(screenMsg: TitleStyle?) {
    setTextColor(Color.parseColor("#" + screenMsg?.fontColor))
    typeface = getFontTypeface(
        screenMsg?.fontBold!!, screenMsg.fontItalic!!,
        screenMsg.fontFamily!!
    )
    val sp: Float =
        screenMsg.fontSize?.replace("px", "")?.toFloat()
            ?: 12f / resources.displayMetrics.scaledDensity + 0.5f

    textSize = sp

    if (screenMsg.fontUnderline!!) {
        paintFlags = Paint.UNDERLINE_TEXT_FLAG
    }
}

/**
 * end screen message style
 */
fun TextView.endScreenMsgStyle(screenMsg: TitleStyle?) {
    setTextColor(Color.parseColor("#" + screenMsg?.fontColor))
    typeface = getFontTypeface(
        screenMsg?.fontBold!!, screenMsg.fontItalic!!,
        screenMsg.fontFamily!!
    )
    val sp: Float =
        screenMsg.fontSize?.replace("px", "")?.toFloat()
            ?: 12f / resources.displayMetrics.scaledDensity + 0.5f

    textSize = sp

    if (screenMsg.fontUnderline!!) {
        paintFlags = Paint.UNDERLINE_TEXT_FLAG
    }
}

/**
 * input answer style
 */
fun TextView.inputAnswerStyle(inputAnswerStyle: TitleStyle?) {
    setTextColor(Color.parseColor("#" + inputAnswerStyle?.fontColor))
    typeface = getFontTypeface(
        inputAnswerStyle?.fontBold!!, inputAnswerStyle.fontItalic!!,
        inputAnswerStyle.fontFamily!!
    )
    val sp: Float =
        inputAnswerStyle.fontSize?.replace("px", "")?.toFloat()
            ?: 12f / resources.displayMetrics.scaledDensity + 0.5f

    textSize = sp

    if (inputAnswerStyle.fontUnderline!!) {
        paintFlags = Paint.UNDERLINE_TEXT_FLAG
    }
}

/**
 * Submit button style
 */
fun MaterialButton.submitButtonStyle(submitButton: SubmitButton?) {
    if (submitButton == null) return
    val style = getFontTypeface(
        submitButton.fontBold!!,
        submitButton.fontItalic!!, submitButton.fontFamily!!
    )
    setTextColor(Color.parseColor("#" + submitButton.fontColor))
    typeface = style
    val sp: Float =
        submitButton.fontSize?.replace("px", "")?.toFloat()
            ?: 12f / resources.displayMetrics.scaledDensity + 0.5f

    textSize = sp
    setBackgroundColor(Color.parseColor("#" + submitButton.backgroundColor))

    if (submitButton.fontUnderline!!) {
        paintFlags = Paint.UNDERLINE_TEXT_FLAG
    }
    val drawable = ContextCompat.getDrawable(context,R.drawable.hve_ic_send)
    setCompoundDrawablesWithIntrinsicBounds(drawable,null,null,null)
    setPadding(64,32,64,32)
    isAllCaps = false
}

fun ImageView.surveyLogoStyle(logoStyle:SurveyLogoStyle){
    Picasso.get().load(logoStyle.uRL)
        .placeholder(R.drawable.loading_animation)
        .resize(45,45)
        .centerInside()
        .into(this)
    alpha = logoStyle.opacity?.toFloat()?: 1f

}

fun Slider.setSliderColor(themeColor: String, focusColor: String){
    this.trackActiveTintList = ColorStateList( arrayOf(intArrayOf(-android.R.attr.state_active)),
        intArrayOf(Color.parseColor("#$focusColor")))
    this.trackInactiveTintList = ColorStateList( arrayOf(intArrayOf(android.R.attr.state_active)),
        intArrayOf(Color.parseColor("#33$themeColor")))
}

fun Spinner.setPopUpColor(themeColor: String,focusColor: String)
{
    val shape = GradientDrawable()
    shape.shape = GradientDrawable.RECTANGLE
    shape.cornerRadius = 20f
    shape.setColor(Color.parseColor("#0D$focusColor"))
    background = shape

}

fun Spinner.setPopupBackgroundDrawable(themeColor: String)
{
    val shape = GradientDrawable()
    shape.shape = GradientDrawable.RECTANGLE
    shape.cornerRadius = 20f
    shape.setColor(Color.parseColor("#40$themeColor"))
    setPopupBackgroundDrawable(shape)
}

// functionality

private fun setTitleStyle(textView: TextView,titleStyle: TitleStyle?){
    textView.setTextColor(Color.parseColor("#" + titleStyle?.fontColor))
    textView.typeface = getFontTypeface(
        titleStyle?.fontBold!!, titleStyle.fontItalic!!,
        titleStyle.fontFamily!!
    )
    val sp: Float =
        titleStyle.fontSize?.replace("px", "")?.toFloat()
            ?: 12f / textView.resources.displayMetrics.scaledDensity + 0.5f

    textView.textSize = sp


    if (titleStyle.fontUnderline!!) {
        textView.paintFlags = Paint.UNDERLINE_TEXT_FLAG
    }
}

////  validation

fun CharSequence?.isValidEmail() =
    !isNullOrEmpty() && Patterns.EMAIL_ADDRESS.matcher(this).matches()

fun String.isValidUrl(): Boolean = URL_PATTERN.toRegex().matches(this)

fun ViewGroup.cardsBackground(context: Context, surveyBackgroundColor: Int) {
    applyColorToDrawable(context,R.drawable.bg_question_rounded,surveyBackgroundColor)
    setBackgroundResource(R.drawable.bg_question_rounded)
}

fun isRtl(context: Context?): Boolean = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
    context?.resources?.configuration?.locales?.get(0)?.language == "ar"
} else {
    context?.resources?.configuration?.locale?.language == "ar"
}
