package com.istnetworks.hivesdk.presentation.surveyExtension

import android.content.Context
import android.graphics.Color
import android.graphics.Typeface
import android.graphics.drawable.Drawable
import android.graphics.drawable.GradientDrawable
import android.graphics.drawable.StateListDrawable
import androidx.annotation.DrawableRes
import androidx.appcompat.content.res.AppCompatResources
import androidx.core.graphics.drawable.DrawableCompat

fun applyColorToDrawable(context: Context,drawableRes: Int, color: Int) {
    val unwrappedDrawable =
        AppCompatResources.getDrawable(context, drawableRes)
    val wrappedDrawable = DrawableCompat.wrap(unwrappedDrawable!!)
    DrawableCompat.setTint(wrappedDrawable, color)
}


fun getFontTypeface(bold: Boolean, italic: Boolean, fontFamily: String): Typeface {
    return if (bold && italic) {
        Typeface.create(fontFamily, Typeface.BOLD_ITALIC)
    } else if (bold && !italic) {
        Typeface.create(fontFamily, Typeface.BOLD)
    } else if (!bold && italic) {
        Typeface.create(fontFamily, Typeface.ITALIC)
    } else {
        Typeface.create(fontFamily, Typeface.NORMAL)
    }
}

fun drawRoundedRectangle(backgroundColor: Int,radius:Float): GradientDrawable? {
    val shape = GradientDrawable()
    shape.shape = GradientDrawable.RECTANGLE
    shape.cornerRadius = radius
    shape.setColor(backgroundColor)
    return shape
}

fun colorStateDrawable(themeColor: String, focusColor: String,radius:Float): StateListDrawable {

    return StateListDrawable().apply {
        addState(
            intArrayOf(-android.R.attr.state_checked),
            drawRoundedRectangle(Color.parseColor("#33$themeColor"),radius))
        addState(intArrayOf(android.R.attr.state_checked),
            drawRoundedRectangle(Color.parseColor("#$focusColor"),radius))
    }
}

fun checkboxColorStateDrawable(normalIcon:Drawable,
                                checkedIcon:Drawable): StateListDrawable {

    return StateListDrawable().apply {
        addState(
            intArrayOf(-android.R.attr.state_checked),
            normalIcon)
        addState(intArrayOf(android.R.attr.state_checked),
            checkedIcon)
    }
}

fun radioButtonColorStateDrawable(normalIcon:Drawable,
                               checkedIcon:Drawable): StateListDrawable {

    return StateListDrawable().apply {
        addState(
            intArrayOf(-android.R.attr.state_checked),
            normalIcon)
        addState(intArrayOf(android.R.attr.state_checked),
            checkedIcon)
    }
}