package com.istnetworks.hivesdk.domain.usecases

import androidx.test.ext.junit.runners.AndroidJUnit4
import com.istnetworks.hivesdk.data.models.Choices
import com.istnetworks.hivesdk.data.models.QuestionResponses
import com.istnetworks.hivesdk.data.models.SelectedChoices
import com.istnetworks.hivesdk.data.models.SkipLogic
import com.istnetworks.hivesdk.data.models.response.Question
import com.istnetworks.hivesdk.data.models.response.Survey
import com.istnetworks.hivesdk.data.utils.HiveSDKType
import com.istnetworks.hivesdk.data.utils.QuestionType
import junit.framework.Assert.assertEquals
import junit.framework.TestCase
import org.junit.Before
import org.junit.Test
import org.junit.runner.RunWith

class SkipLogicHandlerTest  {

    private val questionsList : MutableList<Question> = mutableListOf()
    private val skipList = mutableListOf<SkipLogic>()
    private lateinit var surveyObject: Survey
    private lateinit var skipLogicHandler: SkipLogicHandler
    @Before
     fun setUp() {
        initQuestionList()
        initSkipLogic()
        skipLogicHandler = SkipLogicHandler()
        skipLogicHandler.setData(skipList,questionsList)
    }

    @Test
    fun getSkipToQuestionPosition_hasSkipMultiChoice_Success(){
        val response = QuestionResponses(1,"abc","1",
            null, listOf(
                SelectedChoices(30752,"13e27ea8-0d3b-4d71-990c-a39c41815d8f"),
                SelectedChoices(30751,"4f0bc9fb-66bb-4a47-8b21-a8d52bdaa089")
            ),
            "",null)
        val position = skipLogicHandler.getSkipToQuestionPosition(0,response)

        assertEquals(1,position)
    }
    @Test
    fun getSkipToQuestionPosition_hasSkipRange_Success(){
        val response = QuestionResponses(1,"6a370f7b-075c-4fd8-b535-c6e26f6359fe",null,
            3, null,
            "",null)
        val position = skipLogicHandler.getSkipToQuestionPosition(4,response)

        assertEquals(7,position)
    }

    @Test
    fun getSkipToQuestionPosition_hasSkipRange_Fail(){
        val response = QuestionResponses(1,"6a370f7b-075c-4fd8-b535-c6e26f6359fe",null,
            1, null,
            "",null)
        val position = skipLogicHandler.getSkipToQuestionPosition(4,response)

        assertEquals(-1,position)
    }

    @Test
    fun getSkipToQuestionPosition_noSkip_Success(){
        val response = QuestionResponses(4,"0122d1f3-f549-4e39-927e-b619f807df66"
            ,"04/10/1945",
            null, listOf(),
            "",null)
        val position = skipLogicHandler.getSkipToQuestionPosition(3,response)

        assertEquals(4,position)
    }
    @Test
    fun hasSkipLogic_Exist_success(){
        val hasSkipLogic = skipLogicHandler.hasSkipLogic("abc")
        assertEquals(true,hasSkipLogic)
    }
    @Test
    fun hasSkipLogic_Exist_failed(){
        val hasSkipLogic = skipLogicHandler.hasSkipLogic("0122d1f3-f549-4e39-927e-b619f807df66")
        assertEquals(false,hasSkipLogic)
    }


    private fun initQuestionList(){
        questionsList.add(
            Question("abc",1,"Multiple Choices",
            true,2,
            listOf(
                Choices("13e27ea8-0d3b-4d71-990c-a39c41815d8f",30752,"1","50eca54d",""),
                Choices("4f0bc9fb-66bb-4a47-8b21-a8d52bdaa089",30751,"2","fddb",""),
                Choices("a56c10c1-9541-4c05-8114-d4ccaf443f42",30754,"3","50eca54d","")
            ),
            null)
        )

        questionsList.add(
            Question("9c88051a-223b-4eac-a71c-c8e0d429e931",2,"test input",
            true,9,
            listOf(),
            null)
        )

        questionsList.add(
            Question("96cb2bd9-4402-4438-90df-d5bad3c9213b",3,"single Choices",
            true,15,
            listOf(
                Choices("fb1c62bd-7093-49b3-b6f1-7a6193c4a91c",30750,"test","50eca54d",""),
                Choices("d6225421-ecf3-4958-a48b-cc7770869129",30756,"testt","fddb",""),
                Choices("c7d2d1e4-43c3-46e5-9f8a-ec801c3528d0",30759,"testttt","50eca54d","")
            ),
            null)
        )

        questionsList.add(
            Question("0122d1f3-f549-4e39-927e-b619f807df66",4,"DatePicker",
            false,5,
            listOf(),
            null)
        )
        questionsList.add(Question("6a370f7b-075c-4fd8-b535-c6e26f6359fe",36149,"Sliding Scale "
            ,
            true,QuestionType.SlideQuestion.value,
            listOf(),
            null))
        questionsList.add(
            Question("acf6b89d-b8ae-4e96-81c4-b175f4753e13",5,"Number",
            true,10,
            listOf(),
            null)
        )

        questionsList.add(
            Question("22f06b94-2292-4c13-abc4-634e1c44d472",6,"Email",
            true,11,
            listOf(),
            null)
        )

        questionsList.add(
            Question("ae593422-7634-4e4a-afa8-566c867e4b5e",7,"Phone",
            true,12,
            listOf(),
            null)
        )
        questionsList.add(
            Question("5dfeff9e-5af5-42dc-a2ed-60ee9912012b",8,"URL Question",
            true,14,
            listOf(),
            null)
        )

    }

    private fun initSkipLogic(){
        skipList.add(SkipLogic(id = 20286,questionGUID = "abc",qChoiceGUID = "13e27ea8-0d3b-4d71-990c-a39c41815d8f",minValue = 0,
        maxValue = 0,skipToQuestionGUID = "9c88051a-223b-4eac-a71c-c8e0d429e931"))

        skipList.add(SkipLogic(id = 20288,questionGUID = "96cb2bd9-4402-4438-90df-d5bad3c9213b"
            ,qChoiceGUID = "fb1c62bd-7093-49b3-b6f1-7a6193c4a91c",minValue = 0,
            maxValue = 0,skipToQuestionGUID = "0122d1f3-f549-4e39-927e-b619f807df66"))

        skipList.add(SkipLogic(id = 20289,questionGUID = "22f06b94-2292-4c13-abc4-634e1c44d472"
            ,qChoiceGUID = null,minValue = 0,
            maxValue = 0,skipToQuestionGUID = "5dfeff9e-5af5-42dc-a2ed-60ee9912012b"))

        skipList.add(SkipLogic(id = 20289,questionGUID = "6a370f7b-075c-4fd8-b535-c6e26f6359fe"
            ,qChoiceGUID = null,minValue = 3,
            maxValue = 5,skipToQuestionGUID = "ae593422-7634-4e4a-afa8-566c867e4b5e"))

    }

}