package com.istnetworks.hivesdk.domain.usecases

import androidx.arch.core.executor.testing.InstantTaskExecutorRule
import androidx.test.ext.junit.runners.AndroidJUnit4
import com.istnetworks.hivesdk.data.models.Choices
import com.istnetworks.hivesdk.data.models.QuestionResponses
import com.istnetworks.hivesdk.data.models.SelectedChoices
import com.istnetworks.hivesdk.data.models.response.Question
import junit.framework.TestCase
import org.junit.Before
import org.junit.Rule
import org.junit.Test
import org.junit.rules.TestRule
import org.junit.runner.RunWith
import java.util.*

@RunWith(AndroidJUnit4::class)
class ValidatorTest : TestCase() {
    private val questionsList : MutableList<Question> = mutableListOf()
    private val responseList : MutableList<QuestionResponses> = mutableListOf()
    private lateinit var validator :Validator
    @get:Rule
    var rule: TestRule = InstantTaskExecutorRule()
    @Before
    public override fun setUp(){
        initQuestionList()
        initResponseList()
        validator = Validator()
        validator.setData(questionsList,responseList)
    }

    fun testGetShowIsRequiredErrMsgLD() {}

    fun testGetShowNotValidErrMsgLD() {}
    @Test
    fun testValidateAnswerNotRequiredSuccess() {
        val questionNotRequired = questionsList[3]
        assertEquals(validator.validateAnswer(questionNotRequired,null),true)
    }
    @Test
    fun testValidateAnswerRequiredSuccess() {
        val questionNotRequired = questionsList[4]
        val questionAnswer = responseList[4]
        assertEquals(true,validator.validateAnswer(questionNotRequired,questionAnswer))
    }
    @Test
    fun testValidateAnswerRequiredFail() {
        val questionNotRequired = questionsList[4]
        val result = validator.validateAnswer(questionNotRequired,null)
        assertEquals(false,result)
    }
    @Test
    fun testCheckAllAnswersAreValidSuccess() {
        assertEquals(true, validator.checkAllAnswersAreValid() )
    }

    @Test
    fun testCheckAllAnswersAreValidFail() {
        responseList[5].textResponse = "a@a"
        assertEquals(false , validator.checkAllAnswersAreValid())
    }
    @Test
    fun testCheckIsRequiredAnsweredSuccess() {
        val previousPushed = Stack<Int>()
        previousPushed.push(0)
        previousPushed.push(1)
        previousPushed.push(2)
        previousPushed.push(3)
        previousPushed.push(4)
        previousPushed.push(5)
        previousPushed.push(6)
        previousPushed.push(7)
        previousPushed.push(8)
        previousPushed.push(9)
        previousPushed.push(10)
        previousPushed.push(11)
        assertEquals(true,validator.checkIsRequiredAnswered(previousPushed,10))
    }
    @Test
    fun testCheckIsRequiredAnsweredFail() {
        responseList.removeAt(0)
        val previousPushed = Stack<Int>()
        previousPushed.push(0)
        previousPushed.push(1)
        previousPushed.push(2)
        previousPushed.push(3)
        previousPushed.push(4)
        previousPushed.push(5)
        previousPushed.push(6)
        previousPushed.push(7)
        previousPushed.push(8)
        previousPushed.push(9)
        previousPushed.push(10)
        previousPushed.push(11)
        assertEquals(false,validator.checkIsRequiredAnswered(previousPushed,10))
    }


    private fun initQuestionList(){
        questionsList.add(Question("abc",1,"Multiple Choices",
            true,2,
            listOf(Choices("13e27ea8-0d3b-4d71-990c-a39c41815d8f",30752,"1","50eca54d",""),
                Choices("4f0bc9fb-66bb-4a47-8b21-a8d52bdaa089",30751,"2","fddb",""),
                Choices("a56c10c1-9541-4c05-8114-d4ccaf443f42",30754,"3","50eca54d","")),
            null))

        questionsList.add(Question("9c88051a-223b-4eac-a71c-c8e0d429e931",2,"test input",
            true,9,
            listOf(),
            null))

        questionsList.add(Question("96cb2bd9-4402-4438-90df-d5bad3c9213b",3,"Multiple Choices",
            true,15,
            listOf(Choices("fb1c62bd-7093-49b3-b6f1-7a6193c4a91c",30750,"test","50eca54d",""),
                Choices("d6225421-ecf3-4958-a48b-cc7770869129",30756,"testt","fddb",""),
                Choices("c7d2d1e4-43c3-46e5-9f8a-ec801c3528d0",30759,"testttt","50eca54d","")),
            null))

        questionsList.add(Question("0122d1f3-f549-4e39-927e-b619f807df66",4,"DatePicker",
            false,5,
            listOf(),
            null))

        questionsList.add(Question("acf6b89d-b8ae-4e96-81c4-b175f4753e13",5,"Number",
            true,10,
            listOf(),
            null))

        questionsList.add(Question("22f06b94-2292-4c13-abc4-634e1c44d472",6,"Email",
            true,11,
            listOf(),
            null))

        questionsList.add(Question("ae593422-7634-4e4a-afa8-566c867e4b5e",7,"Phone",
            true,12,
            listOf(),
            null))

        questionsList.add(Question("5dfeff9e-5af5-42dc-a2ed-60ee9912012b",8,"URL Question",
            true,14,
            listOf(),
            null))
    }

    private fun initResponseList(){
        responseList.add(QuestionResponses(1,"abc","1",
        null, listOf(SelectedChoices(30752,"13e27ea8-0d3b-4d71-990c-a39c41815d8f"),
                SelectedChoices(30751,"4f0bc9fb-66bb-4a47-8b21-a8d52bdaa089")),
                "",null))

        responseList.add(QuestionResponses(2,"9c88051a-223b-4eac-a71c-c8e0d429e931"
            ,"test input answer",
            null, listOf(),
            "",null))

        responseList.add(QuestionResponses(3,"96cb2bd9-4402-4438-90df-d5bad3c9213b",
            "test",
            null, listOf(SelectedChoices(30750,"fb1c62bd-7093-49b3-b6f1-7a6193c4a91c")),
            "fb1c62bd-7093-49b3-b6f1-7a6193c4a91c",null))

        responseList.add(QuestionResponses(4,"0122d1f3-f549-4e39-927e-b619f807df66"
            ,"04/10/1945",
            null, listOf(),
            "",null))

        responseList.add(QuestionResponses(5,"acf6b89d-b8ae-4e96-81c4-b175f4753e13"
            ,null,
            123456, listOf(),
            "",null))

        responseList.add(QuestionResponses(6,"22f06b94-2292-4c13-abc4-634e1c44d472"
            ,"a@a.com",
            null, listOf(),
            "",null))

        responseList.add(QuestionResponses(7,"ae593422-7634-4e4a-afa8-566c867e4b5e"
            ,"0123456789",
            null, listOf(),
            "","+20"))

        responseList.add(QuestionResponses(8,"5dfeff9e-5af5-42dc-a2ed-60ee9912012b"
            ,"www.a.com",
            null, listOf(),
            "",null))

    }
}